// *****************************************************************************
//
// Copyright. 2008. Symvionics, Inc. All rights reserved.
//
// Symvionics Inc. ("Symvionics") grants a non-exclusive license for you to use
// this source code and the accompanying documentation, if any, free of charge,
// provided that this paragraph, and the following two paragraphs appear in all
// copies and in all derivative works.  In no event shall this source code or
// this license thereto impact upon or fall within the scope of any other
// agreement that you have with Symvionics.
//
// BY USING THIS SOFTWARE, YOU FURTHER AGREE THAT, IN NO EVENT, SHALL SYMVIONICS
// BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
// SOFTWARE, THE ACCOMPANYING DOCUMENTATION, IF ANY, OR ANY DERIVATIVE WORKS.
// YOU AGREE TO DEFEND, INDEMNIFY, AND HOLD HARMLESS SYMVIONICS FROM AND AGAINST
// ALL FINES, PENALTIES, LIABILITIES, DAMAGES, COSTS, AND EXPENSES INCURRED
// BY SYMVIONICS AS A RESULT OF YOUR USE OF THIS SOFTWARE INCLUDING THIRD PARTY
// USE OF ANY DERIVATIVE WORKS CREATED BY YOU.
//
// SYMVIONICS SPECIFICALLY DISCLAIMS ANY AND ALL WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE.  THE SOFTWARE AND ACCOMPANYING DOCUMENTATION LICENSED
// HEREUNDER ARE PROVIDED "AS IS" WITHOUT WARRANTY OR OBLIGATION OF SUPPORT OR
// MAINTENANCE.
//
// ****************************************************************************

// Written By: Dale Jones
// Date: July 09, 2007
// Email: dale@iads-soft.com
// Google Group: http://groups.google.com/group/iads
//
// IadsDataMove.cpp : Class that reads the names of Tpp parameters from the ParameterDefaults table of
// an IADS config file and copies their corresponding data files to a specified output directory.
//
// Usage: IadsMoveData <configFile> <inputDirectory> <outputDirectory>
//
// <configFile> - The full path to an IADS config file
// <inputDirectory> - The location to copy the parameter data from
// <outputDirectory> - The location to copy the parameter data to

#include "stdafx.h"
#include <atlbase.h>
#include <comutil.h>

char* mConfigFile;
char* mInputDir;
char* mOutputDir;

IDispatch* mConfigObj;
BSTR* mParamNames;
int mNumParams;

/**
 * parseCommandLine - Parses the command line arguments
 * 
 * Parameters:
 *    argv[] - An array of command line arguments
 */
void parseCommandLine(char* argv[])
{
    mConfigFile = new char[strlen(argv[1]) + 1];
    mInputDir = new char[strlen(argv[2]) + 1];
    mOutputDir = new char[strlen(argv[3]) + 1];

    strcpy(mConfigFile, argv[1]);
    strcpy(mInputDir, argv[2]);
    strcpy(mOutputDir, argv[3]);
}

/**
 * FileExists - Determines if a file/folder exists at the specified location
 * 
 * Parameters:
 *    fileName - Path to the file to check
 * 
 * Return:
 *    True if the file exists, false if it doesn't
 */
BOOL FileExists(char* fileName)
{
    DWORD dwAttr = GetFileAttributes(fileName);
	if (dwAttr == 0xffffffff)
		return FALSE;
	else 
		return TRUE;
}

/**
 * CleanUp - Frees memory used by this class' member variables
 */
void CleanUp()
{
    if (mConfigFile) delete[] mConfigFile;
    if (mInputDir) delete[] mInputDir;
    if (mOutputDir) delete[] mOutputDir;
}

/**
 * OpenConfigFile - Opens an IADS config file using the IadsConfigInterface
 * 
 * Return:
 *    The return status of the open process. 
 *    0 indicates success, negative number means failure.
 */
HRESULT OpenConfigFile()
{
    CComBSTR configFile = _T(mConfigFile);

    // Set up parameters for call to "Open"
    VARIANT params[1];
    VARIANT vtResult;

    ZeroMemory(params, sizeof(params));
    params[0].vt = VT_BSTR;
    params[0].bstrVal = configFile;

    DISPPARAMS openParams;
    openParams.cArgs = 1;
    openParams.cNamedArgs = 0;
    openParams.rgdispidNamedArgs = NULL;
    openParams.rgvarg = params;
    
    VariantInit(&vtResult);

    // Make the call to the IadsConfigInterface
    DISPID openID;
    OLECHAR FAR* szMember = L"Open";
    mConfigObj->GetIDsOfNames(IID_NULL, &szMember, 1, LOCALE_SYSTEM_DEFAULT, &openID);
    if (mConfigObj->Invoke(openID, IID_NULL, LOCALE_USER_DEFAULT, DISPATCH_METHOD, &openParams, &vtResult, NULL, NULL) != S_OK)
    {
        printf("IadsDataMove::OpenConfigFile() - Error loading config file! Verify that the config is not corrupt or invalid.\n");
        return -1;
    }

    // Cleanup memory
    VariantClear(&params[0]);
    VariantClear(&vtResult);
    return S_OK;
}

/**
 * CloseConfigFile - Closes the IADS config file
 * 
 * Return:
 *    The return status of the close process. 
 *    0 indicates success, negative number means failure.
 */
HRESULT CloseConfigFile()
{
    VARIANT vtResult;
    VariantInit(&vtResult);

    // "Close" function takes no parameters
    DISPPARAMS dispparamsNoArgs = {NULL, NULL, 0, 0};

    // Make the call to the IadsConfigInterface
    DISPID closeID;
    OLECHAR FAR* szMember = L"Close";
    mConfigObj->GetIDsOfNames(IID_NULL, &szMember, 1, LOCALE_SYSTEM_DEFAULT, &closeID);
    if (mConfigObj->Invoke(closeID, IID_NULL, LOCALE_USER_DEFAULT, DISPATCH_METHOD, &dispparamsNoArgs, &vtResult, NULL, NULL) != S_OK)
    {
        printf("IadsDataMove::CloseConfigFile() - Error closing config file!\n");
        return -1;
    }

    // Cleanup memory
    VariantClear(&vtResult);

    return S_OK;
}

/**
 * InitIadsConfigInterface - Initializes the IadsConfigInterface object
 * 
 * Return:
 *    The return status of the initialization process. 
 *    0 indicates success, negative number means failure.
 */
HRESULT InitIadsConfigInterface()
{
    CLSID clsid;
    HRESULT hr = ::CLSIDFromProgID( L"IadsConfigInterface.IadsConfig", &clsid);
    if (hr != S_OK) 
    {
        printf("IadsDataMove::InitIadsConfigInterface() - Error getting CLSID for IadsConfigInterface.IadsConfig. Make sure you have the DLL registered on this machine.\n");
        return -1;
    }
    
    hr = ::CoCreateInstance(clsid, NULL, CLSCTX_ALL, IID_IUnknown, (void**)&mConfigObj);
    if (hr != S_OK) 
    {
        printf("IadsDataMove::InitIadsConfigInterface() - Error CoCreating IadsConfigInterface. Make sure you have the DLL registered on this machine.\n");
        return -1;
    }

    return S_OK;
}

/**
 * GetParameterNames - Retrieves the list of Tpp parameter names from an IADS config file.
 * 
 * Return:
 *    The return status of the process. 
 *    0 indicates success, negative number means failure.
 */
HRESULT GetParameterNames()
{
    // Set up parameters for call to "Query"
    VARIANT params[1];
    VARIANT vtResult;

    ZeroMemory(params, sizeof(params));
    params[0].vt = VT_BSTR;
    params[0].bstrVal = L"select Parameter from ParameterDefaults where DataSourceType = 'Tpp'";

    DISPPARAMS queryParams;
    queryParams.cArgs = 1;
    queryParams.cNamedArgs = 0;
    queryParams.rgdispidNamedArgs = NULL;
    queryParams.rgvarg = params;
    
    VariantInit(&vtResult);

    // Make the call to the IadsConfigInterface
    DISPID queryID;
    OLECHAR FAR* szMember = L"Query";
    mConfigObj->GetIDsOfNames(IID_NULL, &szMember, 1, LOCALE_SYSTEM_DEFAULT, &queryID);

    if (mConfigObj->Invoke(queryID, IID_NULL, LOCALE_USER_DEFAULT, DISPATCH_METHOD, &queryParams, &vtResult, NULL, NULL) != S_OK)
    {
        printf("IadsDataMove::GetParameterNames() - Error getting parameter names from the config file! Verify that the config is not corrupt or invalid.\n");
        return -1;
    }

    // vtResult should contain an array of parameter names now
    SafeArrayAccessData( vtResult.parray,(void**)&mParamNames );
    int lBound = vtResult.parray->rgsabound->lLbound;
    int uBound = vtResult.parray->rgsabound->cElements + lBound;
    mNumParams = uBound - lBound;

    // Cleanup memory
    VariantClear(&vtResult);

    return S_OK;
}

/**
 * CopyParameterData - Copies the parameter data files from the input directory to the output directory.
 * 
 * Return:
 *    The return status of the copy process. 
 *    0 indicates success, negative number means failure.
 */
HRESULT CopyParameterData()
{
    bool someFailed = false;
    char inputPath[20000];
    char outputPath[20000];

    USES_CONVERSION;
    char* tempName;

    for (register int i = 0; i < mNumParams; i++)
    {
        tempName = OLE2T(mParamNames[i]);
        sprintf(inputPath, "%s\\%s.iadsData", mInputDir, tempName);

        if (!FileExists(inputPath))
        {
            printf("No parameter data found for parameter \"%s\"\n", tempName);
            continue;
        }

        sprintf(outputPath, "%s\\%s.iadsData", mOutputDir, tempName);
        if (::CopyFile(inputPath, outputPath, false))
        {
            printf("Successfully copied data file \"%s\" to \"%s\".\n", tempName, outputPath);
        }
        else
        {
            printf("Failed to copy data file \"%s\" to \"%s\".\n", inputPath, outputPath);
        }
    }

    return S_OK;
}

/**
 * main - Entry point into the IadsDataMove class
 */
int main(int argc, char* argv[])
{
	if (argc != 4)
    {
        printf("Usage: IADSDataMove <configFile> <inputDirectory> <outputDirectory>\n");
        return -1;
    }

    parseCommandLine(argv);

    if (!FileExists(mConfigFile))
    {
        printf("ERROR - Specified config file \"%s\" does not exist!\n", mConfigFile);
        CleanUp();
        return -1;
    }

    if (!FileExists(mInputDir))
    {
        printf("ERROR - Specified input directory \"%s\" does not exist!\n", mInputDir);
        CleanUp();
        return -1;
    }

    if (!FileExists(mOutputDir))
    {
        printf("ERROR - Specified output directory \"%s\" does not exist!\n", mOutputDir);
        CleanUp();
        return -1;
    }

    ::CoInitialize(NULL);

    HRESULT hr = InitIadsConfigInterface();
    if (hr != S_OK)
    {
        CleanUp();
        return -1;
    }

    hr = OpenConfigFile();
    if (hr != S_OK)
    {
        CleanUp();
        return -1;
    }

    hr = GetParameterNames();
    if (hr != S_OK)
    {
        CleanUp();
        return -1;
    }

    CopyParameterData();
    CloseConfigFile();
    
    CoUninitialize(); 
    CleanUp();
    
    return 0;
}
